@@ -2,21 +2,19 @@
  * history.c
  * Copyright 2011 John Lindgren
  *
- * This file is part of Audacious.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * Audacious is free software: you can redistribute it and/or modify it under
- * the terms of the GNU General Public License as published by the Free Software
- * Foundation, version 2 or version 3 of the License.
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions, and the following disclaimer.
  *
- * Audacious is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
- * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions, and the following disclaimer in the documentation
+ *    provided with the distribution.
  *
- * You should have received a copy of the GNU General Public License along with
- * Audacious. If not, see <http://www.gnu.org/licenses/>.
- *
- * The Audacious team does not consider modular code linking to Audacious or
- * using our public API to be a derived work.
+ * This software is provided "as is" and without any warranty, express or
+ * implied. In no event shall the authors be liable for any damages arising from
+ * the use of this software.
  */
 
 #include <glib.h>
@@ -38,8 +36,6 @@ static void history_save (void)
     if (! modified)
         return;
 
-    config_clear_section ("history");
-
     GList * node = history.head;
     for (int i = 0; i < MAX_ENTRIES; i ++)
     {
@@ -48,7 +44,7 @@ static void history_save (void)
 
         char name[32];
         snprintf (name, sizeof name, "entry%d", i);
-        set_string ("history", name, node->data);
+        set_str ("history", name, node->data);
 
         node = node->next;
     }
@@ -65,11 +61,11 @@ static void history_load (void)
     {
         char name[32];
         snprintf (name, sizeof name, "entry%d", i);
-        char * path = get_string ("history", name);
+        char * path = get_str ("history", name);
 
         if (! path[0])
         {
-            g_free (path);
+            str_unref (path);
             break;
         }
 
@@ -87,7 +83,7 @@ void history_cleanup (void)
 
     hook_dissociate ("config save", (HookFunction) history_save);
 
-    g_queue_foreach (& history, (GFunc) g_free, NULL);
+    g_queue_foreach (& history, (GFunc) str_unref, NULL);
     g_queue_clear (& history);
 
     loaded = FALSE;
@@ -110,11 +106,11 @@ void history_add (const char * path)
         next = node->next;
         if (! strcmp (node->data, path))
         {
-            g_free (node->data);
+            str_unref (node->data);
             g_queue_delete_link (& history, node);
         }
     }
 
-    g_queue_push_head (& history, g_strdup (path));
+    g_queue_push_head (& history, str_get (path));
     modified = TRUE;
 }
