@@ -1,6 +1,6 @@
 /*
  * interface.c
- * Copyright 2010-2011 John Lindgren
+ * Copyright 2010-2013 John Lindgren
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -21,21 +21,23 @@
 #include <pthread.h>
 
 #include <libaudcore/hook.h>
+#include <libaudgui/libaudgui-gtk.h>
 
 #include "debug.h"
 #include "general.h"
+#include "i18n.h"
 #include "interface.h"
-#include "main.h"
 #include "misc.h"
 #include "plugin.h"
 #include "plugins.h"
 #include "visualization.h"
 
-static IfacePlugin *current_interface = NULL;
+static IfacePlugin * current_interface = NULL;
 
 static pthread_mutex_t error_mutex = PTHREAD_MUTEX_INITIALIZER;
 static GQueue error_queue = G_QUEUE_INIT;
 static int error_source;
+static GtkWidget * error_win;
 
 bool_t interface_load (PluginHandle * plugin)
 {
@@ -73,18 +75,7 @@ bool_t interface_is_shown (void)
 {
     g_return_val_if_fail (current_interface, FALSE);
 
-    if (PLUGIN_HAS_FUNC (current_interface, is_shown))
-        return current_interface->is_shown ();
-    return TRUE;
-}
-
-bool_t interface_is_focused (void)
-{
-    g_return_val_if_fail (current_interface, FALSE);
-
-    if (PLUGIN_HAS_FUNC (current_interface, is_focused))
-        return current_interface->is_focused ();
-    return TRUE;
+    return get_bool (NULL, "show_interface");
 }
 
 static bool_t error_idle_func (void * unused)
@@ -96,12 +87,12 @@ static bool_t error_idle_func (void * unused)
     {
         pthread_mutex_unlock (& error_mutex);
 
-        if (current_interface && PLUGIN_HAS_FUNC (current_interface, show_error))
-            current_interface->show_error (message);
-        else
+        if (headless_mode ())
             fprintf (stderr, "ERROR: %s\n", message);
+        else
+            audgui_simple_message (& error_win, GTK_MESSAGE_ERROR, _("Error"), message);
 
-        g_free (message);
+        str_unref (message);
 
         pthread_mutex_lock (& error_mutex);
     }
@@ -116,7 +107,7 @@ void interface_show_error (const char * message)
 {
     pthread_mutex_lock (& error_mutex);
 
-    g_queue_push_tail (& error_queue, g_strdup (message));
+    g_queue_push_tail (& error_queue, str_get (message));
 
     if (! error_source)
         error_source = g_idle_add (error_idle_func, NULL);
@@ -124,27 +115,6 @@ void interface_show_error (const char * message)
     pthread_mutex_unlock (& error_mutex);
 }
 
-/*
- * bool_t play_button
- *       TRUE  - open files
- *       FALSE - add files
- */
-void interface_show_filebrowser (bool_t play_button)
-{
-    g_return_if_fail (current_interface);
-
-    if (PLUGIN_HAS_FUNC (current_interface, show_filebrowser))
-        current_interface->show_filebrowser (play_button);
-}
-
-void interface_show_jump_to_track (void)
-{
-    g_return_if_fail (current_interface);
-
-    if (PLUGIN_HAS_FUNC (current_interface, show_jump_to_track))
-        current_interface->show_jump_to_track ();
-}
-
 static bool_t delete_cb (GtkWidget * window, GdkEvent * event, PluginHandle *
  plugin)
 {
@@ -180,26 +150,6 @@ void interface_remove_plugin_widget (PluginHandle * plugin, GtkWidget * widget)
         gtk_widget_destroy (gtk_widget_get_parent (widget));
 }
 
-void interface_install_toolbar (void * widget)
-{
-    g_return_if_fail (current_interface);
-
-    if (PLUGIN_HAS_FUNC (current_interface, install_toolbar))
-        current_interface->install_toolbar (widget);
-    else
-        g_object_ref (widget);
-}
-
-void interface_uninstall_toolbar (void * widget)
-{
-    g_return_if_fail (current_interface);
-
-    if (PLUGIN_HAS_FUNC (current_interface, uninstall_toolbar))
-        current_interface->uninstall_toolbar (widget);
-    else
-        g_object_unref (widget);
-}
-
 static bool_t probe_cb (PluginHandle * p, PluginHandle * * pp)
 {
     * pp = p;
