@@ -1,6 +1,6 @@
 /*
  * plugin-registry.c
- * Copyright 2009-2011 John Lindgren
+ * Copyright 2009-2013 John Lindgren
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -24,11 +24,13 @@
  * loaded; hence the mutex must be locked before checking that a plugin is
  * loaded and while loading it. */
 
-#include <glib.h>
 #include <pthread.h>
 #include <stdio.h>
 #include <string.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include <libaudcore/audstrings.h>
 
 #include "debug.h"
@@ -99,7 +101,7 @@ static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 static PluginHandle * plugin_new (char * path, bool_t confirmed, bool_t
  loaded, int timestamp, int type, Plugin * header)
 {
-    PluginHandle * plugin = g_malloc (sizeof (PluginHandle));
+    PluginHandle * plugin = g_slice_new (PluginHandle);
 
     plugin->path = path;
     plugin->confirmed = confirmed;
@@ -144,31 +146,31 @@ static void plugin_free (PluginHandle * plugin)
 {
     plugin_list = g_list_remove (plugin_list, plugin);
 
-    g_list_free_full (plugin->watches, g_free);
+    for (GList * node = plugin->watches; node; node = node->next)
+        g_slice_free (PluginWatch, node->data);
 
     if (plugin->type == PLUGIN_TYPE_TRANSPORT)
-        g_list_free_full (plugin->u.t.schemes, g_free);
+        g_list_free_full (plugin->u.t.schemes, (GDestroyNotify) str_unref);
     else if (plugin->type == PLUGIN_TYPE_PLAYLIST)
-        g_list_free_full (plugin->u.p.exts, g_free);
+        g_list_free_full (plugin->u.p.exts, (GDestroyNotify) str_unref);
     else if (plugin->type == PLUGIN_TYPE_INPUT)
     {
         for (int key = 0; key < INPUT_KEYS; key ++)
-            g_list_free_full (plugin->u.i.keys[key], g_free);
+            g_list_free_full (plugin->u.i.keys[key], (GDestroyNotify) str_unref);
     }
 
-    g_free (plugin->path);
-    g_free (plugin->name);
-    g_free (plugin->domain);
+    str_unref (plugin->path);
+    str_unref (plugin->name);
+    str_unref (plugin->domain);
     g_free (plugin->misc);
-    g_free (plugin);
+    g_slice_free (PluginHandle, plugin);
 }
 
 static FILE * open_registry_file (const char * mode)
 {
-    char * path = g_strdup_printf ("%s/" FILENAME, get_path (AUD_PATH_USER_DIR));
-    FILE * file = fopen (path, mode);
-    g_free (path);
-    return file;
+    const char * user_dir = get_path (AUD_PATH_USER_DIR);
+    SCONCAT2 (path, user_dir, "/" FILENAME);
+    return g_fopen (path, mode);
 }
 
 static void transport_plugin_save (PluginHandle * plugin, FILE * handle)
@@ -264,8 +266,7 @@ static bool_t parse_integer (const char * key, int * value)
 
 static char * parse_string (const char * key)
 {
-    return (parse_value && ! strcmp (parse_key, key)) ? g_strdup (parse_value) :
-     NULL;
+    return (parse_value && ! strcmp (parse_key, key)) ? str_get (parse_value) : NULL;
 }
 
 static void transport_plugin_parse (PluginHandle * plugin, FILE * handle)
@@ -330,7 +331,7 @@ FOUND:
     int timestamp;
     if (! parse_integer ("stamp", & timestamp))
     {
-        g_free (path);
+        str_unref (path);
         return FALSE;
     }
 
@@ -405,10 +406,10 @@ int plugin_compare (PluginHandle * a, PluginHandle * b)
         return 1;
 
     int diff;
-    if ((diff = string_compare (dgettext (a->domain, a->name), dgettext (b->domain, b->name))))
+    if ((diff = str_compare (dgettext (a->domain, a->name), dgettext (b->domain, b->name))))
         return diff;
 
-    return string_compare (a->path, b->path);
+    return str_compare (a->path, b->path);
 }
 
 void plugin_registry_prune (void)
@@ -432,16 +433,15 @@ PluginHandle * plugin_lookup (const char * path)
 
 static int plugin_lookup_basename_cb (PluginHandle * plugin, const char * basename)
 {
-    char * test = g_path_get_basename (plugin->path);
+    const char * slash = strrchr (plugin->path, G_DIR_SEPARATOR);
+    if (! slash)
+        return TRUE;
 
-    char * dot = strrchr (test, '.');
-    if (dot)
-        * dot = 0;
+    const char * dot = strrchr (slash + 1, '.');
+    if (! dot)
+        return TRUE;
 
-    int ret = strcmp (test, basename);
-
-    g_free (test);
-    return ret;
+    return strncmp (slash + 1, basename, dot - (slash + 1));
 }
 
 /* Note: If there are multiple plugins with the same basename, this returns only
@@ -457,10 +457,10 @@ static void plugin_get_info (PluginHandle * plugin, bool_t new)
 {
     Plugin * header = plugin->header;
 
-    g_free (plugin->name);
-    g_free (plugin->domain);
-    plugin->name = g_strdup (header->name);
-    plugin->domain = PLUGIN_HAS_FUNC (header, domain) ? g_strdup (header->domain) : NULL;
+    str_unref (plugin->name);
+    str_unref (plugin->domain);
+    plugin->name = str_get (header->name);
+    plugin->domain = PLUGIN_HAS_FUNC (header, domain) ? str_get (header->domain) : NULL;
     plugin->has_about = PLUGIN_HAS_FUNC (header, about) || PLUGIN_HAS_FUNC (header, about_text);
     plugin->has_configure = PLUGIN_HAS_FUNC (header, configure) || PLUGIN_HAS_FUNC (header, prefs);
 
@@ -468,23 +468,21 @@ static void plugin_get_info (PluginHandle * plugin, bool_t new)
     {
         TransportPlugin * tp = (TransportPlugin *) header;
 
-        g_list_free_full (plugin->u.t.schemes, g_free);
+        g_list_free_full (plugin->u.t.schemes, (GDestroyNotify) str_unref);
         plugin->u.t.schemes = NULL;
 
         for (int i = 0; tp->schemes[i]; i ++)
-            plugin->u.t.schemes = g_list_prepend (plugin->u.t.schemes, g_strdup
-             (tp->schemes[i]));
+            plugin->u.t.schemes = g_list_prepend (plugin->u.t.schemes, str_get (tp->schemes[i]));
     }
     else if (header->type == PLUGIN_TYPE_PLAYLIST)
     {
         PlaylistPlugin * pp = (PlaylistPlugin *) header;
 
-        g_list_free_full (plugin->u.p.exts, g_free);
+        g_list_free_full (plugin->u.p.exts, (GDestroyNotify) str_unref);
         plugin->u.p.exts = NULL;
 
         for (int i = 0; pp->extensions[i]; i ++)
-            plugin->u.p.exts = g_list_prepend (plugin->u.p.exts, g_strdup
-             (pp->extensions[i]));
+            plugin->u.p.exts = g_list_prepend (plugin->u.p.exts, str_get (pp->extensions[i]));
     }
     else if (header->type == PLUGIN_TYPE_INPUT)
     {
@@ -493,7 +491,7 @@ static void plugin_get_info (PluginHandle * plugin, bool_t new)
 
         for (int key = 0; key < INPUT_KEYS; key ++)
         {
-            g_list_free_full (plugin->u.i.keys[key], g_free);
+            g_list_free_full (plugin->u.i.keys[key], (GDestroyNotify) str_unref);
             plugin->u.i.keys[key] = NULL;
         }
 
@@ -501,22 +499,22 @@ static void plugin_get_info (PluginHandle * plugin, bool_t new)
         {
             for (int i = 0; ip->extensions[i]; i ++)
                 plugin->u.i.keys[INPUT_KEY_EXTENSION] = g_list_prepend
-                 (plugin->u.i.keys[INPUT_KEY_EXTENSION], g_strdup
-                 (ip->extensions[i]));
+                 (plugin->u.i.keys[INPUT_KEY_EXTENSION],
+                 str_get (ip->extensions[i]));
         }
 
         if (PLUGIN_HAS_FUNC (ip, mimes))
         {
             for (int i = 0; ip->mimes[i]; i ++)
                 plugin->u.i.keys[INPUT_KEY_MIME] = g_list_prepend
-                 (plugin->u.i.keys[INPUT_KEY_MIME], g_strdup (ip->mimes[i]));
+                 (plugin->u.i.keys[INPUT_KEY_MIME], str_get (ip->mimes[i]));
         }
 
         if (PLUGIN_HAS_FUNC (ip, schemes))
         {
             for (int i = 0; ip->schemes[i]; i ++)
                 plugin->u.i.keys[INPUT_KEY_SCHEME] = g_list_prepend
-                 (plugin->u.i.keys[INPUT_KEY_SCHEME], g_strdup (ip->schemes[i]));
+                 (plugin->u.i.keys[INPUT_KEY_SCHEME], str_get (ip->schemes[i]));
         }
 
         plugin->u.i.has_images = PLUGIN_HAS_FUNC (ip, get_song_image);
@@ -572,7 +570,7 @@ void plugin_register (const char * path, int timestamp)
         if (! header)
             return;
 
-        plugin = plugin_new (g_strdup (path), TRUE, TRUE, timestamp,
+        plugin = plugin_new (str_get (path), TRUE, TRUE, timestamp,
          header->type, header);
 
         plugin_get_info (plugin, TRUE);
@@ -620,6 +618,54 @@ PluginHandle * plugin_by_header (const void * header)
     return node ? node->data : NULL;
 }
 
+int plugin_count (int type)
+{
+    int count = 0;
+
+    for (GList * node = plugin_list; node; node = node->next)
+    {
+        PluginHandle * plugin = node->data;
+        if (plugin->type == type)
+            count ++;
+    }
+
+    return count;
+}
+
+int plugin_get_index (PluginHandle * plugin)
+{
+    int index = 0;
+
+    for (GList * node = plugin_list; node; node = node->next)
+    {
+        PluginHandle * plugin2 = node->data;
+        if (plugin2->type == plugin->type)
+        {
+            if (plugin2 == plugin)
+                return index;
+            index ++;
+        }
+    }
+
+    return -1;
+}
+
+PluginHandle * plugin_by_index (int type, int index)
+{
+    for (GList * node = plugin_list; node; node = node->next)
+    {
+        PluginHandle * plugin = node->data;
+        if (plugin->type == type)
+        {
+            if (! index)
+                return plugin;
+            index --;
+        }
+    }
+
+    return NULL;
+}
+
 void plugin_for_each (int type, PluginForEachFunc func, void * data)
 {
     for (GList * node = plugin_list; node; node = node->next)
@@ -660,7 +706,7 @@ static void plugin_call_watches (PluginHandle * plugin)
 
         if (! watch->func (plugin, watch->data))
         {
-            g_free (watch);
+            g_slice_free (PluginWatch, watch);
             plugin->watches = g_list_delete_link (plugin->watches, node);
         }
 
@@ -696,7 +742,7 @@ void plugin_for_enabled (int type, PluginForEachFunc func, void * data)
 void plugin_add_watch (PluginHandle * plugin, PluginForEachFunc func, void *
  data)
 {
-    PluginWatch * watch = g_malloc (sizeof (PluginWatch));
+    PluginWatch * watch = g_slice_new (PluginWatch);
     watch->func = func;
     watch->data = data;
     plugin->watches = g_list_prepend (plugin->watches, watch);
@@ -712,7 +758,7 @@ void plugin_remove_watch (PluginHandle * plugin, PluginForEachFunc func, void *
 
         if (watch->func == func && watch->data == data)
         {
-            g_free (watch);
+            g_slice_free (PluginWatch, watch);
             plugin->watches = g_list_delete_link (plugin->watches, node);
         }
 
@@ -754,7 +800,8 @@ PluginHandle * transport_plugin_for_scheme (const char * scheme)
 
 typedef struct {
     const char * ext;
-    PluginHandle * plugin;
+    PluginForEachFunc func;
+    void * data;
 } PlaylistPluginForExtState;
 
 static bool_t playlist_plugin_for_ext_cb (PluginHandle * plugin,
@@ -764,16 +811,14 @@ static bool_t playlist_plugin_for_ext_cb (PluginHandle * plugin,
      (GCompareFunc) g_ascii_strcasecmp))
         return TRUE;
 
-    state->plugin = plugin;
-    return FALSE;
+    return state->func (plugin, state->data);
 }
 
-PluginHandle * playlist_plugin_for_extension (const char * extension)
+void playlist_plugin_for_ext (const char * ext, PluginForEachFunc func, void * data)
 {
-    PlaylistPluginForExtState state = {extension, NULL};
+    PlaylistPluginForExtState state = {ext, func, data};
     plugin_for_enabled (PLUGIN_TYPE_PLAYLIST, (PluginForEachFunc)
      playlist_plugin_for_ext_cb, & state);
-    return state.plugin;
 }
 
 typedef struct {
