@@ -1,6 +1,6 @@
 /*
  * tuple.c
- * Copyright 2007-2011 William Pitcock, Christian Birchinger, Matti Hämäläinen,
+ * Copyright 2007-2013 William Pitcock, Christian Birchinger, Matti Hämäläinen,
  *                     Giacomo Lozito, Eugene Zagidullin, and John Lindgren
  *
  * Redistribution and use in source and binary forms, with or without
@@ -18,13 +18,7 @@
  * the use of this software.
  */
 
-/**
- * @file tuple.c
- * @brief Basic Tuple handling API.
- */
-
 #include <glib.h>
-#include <pthread.h>
 #include <stdio.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -33,8 +27,12 @@
 #include <audacious/i18n.h>
 
 #include "audstrings.h"
+#include "tinylock.h"
 #include "tuple.h"
-#include "tuple_formatter.h"
+
+#if TUPLE_FIELDS > 64
+#error The current tuple implementation is limited to 64 fields
+#endif
 
 #define BLOCK_VALS 4
 
@@ -61,20 +59,25 @@ struct _TupleBlock {
  * metadata. This is not the same as a playlist entry, though.
  */
 struct _Tuple {
-    int refcount;
     int64_t setmask;
     TupleBlock * blocks;
 
-    int nsubtunes;                 /**< Number of subtunes, if any. Values greater than 0
-                                         mean that there are subtunes and #subtunes array
-                                         may be set. */
     int *subtunes;                 /**< Array of int containing subtune index numbers.
                                          Can be NULL if indexing is linear or if
                                          there are no subtunes. */
+    int nsubtunes;                 /**< Number of subtunes, if any. Values greater than 0
+                                         mean that there are subtunes and #subtunes array
+                                         may be set. */
+
+    int refcount;
+    TinyLock lock;
 };
 
 #define BIT(i) ((int64_t) 1 << (i))
 
+#define LOCK(t) tiny_lock ((TinyLock *) & t->lock)
+#define UNLOCK(t) tiny_unlock ((TinyLock *) & t->lock)
+
 /** Ordered table of basic #Tuple field names and their #TupleValueType.
  */
 static const TupleBasicType tuple_fields[TUPLE_FIELDS] = {
@@ -154,8 +157,8 @@ static const FieldDictEntry field_dict[TUPLE_FIELDS] = {
  {"track-number", FIELD_TRACK_NUMBER},
  {"year", FIELD_YEAR}};
 
-static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-
+#define VALID_FIELD(f) ((f) >= 0 && (f) < TUPLE_FIELDS)
+#define FIELD_TYPE(f) (tuple_fields[f].type)
 
 static int field_dict_compare (const void * a, const void * b)
 {
@@ -168,26 +171,18 @@ EXPORT int tuple_field_by_name (const char * name)
     FieldDictEntry * found = bsearch (& find, field_dict, TUPLE_FIELDS,
      sizeof (FieldDictEntry), field_dict_compare);
 
-    if (found)
-        return found->field;
-
-    fprintf (stderr, "Unknown tuple field name \"%s\".\n", name);
-    return -1;
+    return found ? found->field : -1;
 }
 
 EXPORT const char * tuple_field_get_name (int field)
 {
-    if (field < 0 || field >= TUPLE_FIELDS)
-        return NULL;
-
+    g_return_val_if_fail (VALID_FIELD (field), NULL);
     return tuple_fields[field].name;
 }
 
 EXPORT TupleValueType tuple_field_get_type (int field)
 {
-    if (field < 0 || field >= TUPLE_FIELDS)
-        return TUPLE_UNKNOWN;
-
+    g_return_val_if_fail (VALID_FIELD (field), TUPLE_UNKNOWN);
     return tuple_fields[field].type;
 }
 
@@ -240,7 +235,7 @@ static TupleVal * lookup_val (Tuple * tuple, int field, bool_t add, bool_t remov
     return & block->vals[0];
 }
 
-static void tuple_destroy_unlocked (Tuple * tuple)
+static void tuple_destroy (Tuple * tuple)
 {
     TupleBlock * next;
     for (TupleBlock * block = tuple->blocks; block; block = next)
@@ -254,13 +249,10 @@ static void tuple_destroy_unlocked (Tuple * tuple)
                 str_unref (block->vals[i].str);
         }
 
-        memset (block, 0, sizeof (TupleBlock));
         g_slice_free (TupleBlock, block);
     }
 
-    g_free(tuple->subtunes);
-
-    memset (tuple, 0, sizeof (Tuple));
+    g_free (tuple->subtunes);
     g_slice_free (Tuple, tuple);
 }
 
@@ -273,11 +265,8 @@ EXPORT Tuple * tuple_new (void)
 
 EXPORT Tuple * tuple_ref (Tuple * tuple)
 {
-    pthread_mutex_lock (& mutex);
+    __sync_fetch_and_add (& tuple->refcount, 1);
 
-    tuple->refcount ++;
-
-    pthread_mutex_unlock (& mutex);
     return tuple;
 }
 
@@ -286,22 +275,10 @@ EXPORT void tuple_unref (Tuple * tuple)
     if (! tuple)
         return;
 
-    pthread_mutex_lock (& mutex);
-
-    if (! -- tuple->refcount)
-        tuple_destroy_unlocked (tuple);
-
-    pthread_mutex_unlock (& mutex);
+    if (! __sync_sub_and_fetch (& tuple->refcount, 1))
+        tuple_destroy (tuple);
 }
 
-/**
- * Sets filename/URI related fields of a #Tuple structure, based
- * on the given filename argument. The fields set are:
- * #FIELD_FILE_PATH, #FIELD_FILE_NAME and #FIELD_FILE_EXT.
- *
- * @param[in] filename Filename URI.
- * @param[in,out] tuple Tuple structure to manipulate.
- */
 EXPORT void tuple_set_filename (Tuple * tuple, const char * filename)
 {
     const char * base, * ext, * sub;
@@ -311,34 +288,27 @@ EXPORT void tuple_set_filename (Tuple * tuple, const char * filename)
 
     char path[base - filename + 1];
     str_decode_percent (filename, base - filename, path);
-    tuple_set_str (tuple, FIELD_FILE_PATH, NULL, path);
+    tuple_set_str (tuple, FIELD_FILE_PATH, path);
 
     char name[ext - base + 1];
     str_decode_percent (base, ext - base, name);
-    tuple_set_str (tuple, FIELD_FILE_NAME, NULL, name);
+    tuple_set_str (tuple, FIELD_FILE_NAME, name);
 
     if (ext < sub)
     {
         char extbuf[sub - ext];
         str_decode_percent (ext + 1, sub - ext - 1, extbuf);
-        tuple_set_str (tuple, FIELD_FILE_EXT, NULL, extbuf);
+        tuple_set_str (tuple, FIELD_FILE_EXT, extbuf);
     }
 
     if (sub[0])
-        tuple_set_int (tuple, FIELD_SUBSONG_ID, NULL, isub);
+        tuple_set_int (tuple, FIELD_SUBSONG_ID, isub);
 }
 
-/**
- * Creates a copy of given Tuple structure, with copied data.
- *
- * @param[in] src Tuple structure to be made a copy of.
- * @return Pointer to newly allocated Tuple.
- */
 EXPORT Tuple * tuple_copy (const Tuple * old)
 {
-    pthread_mutex_lock (& mutex);
-
     Tuple * new = tuple_new ();
+    LOCK (old);
 
     for (int f = 0; f < TUPLE_FIELDS; f ++)
     {
@@ -358,83 +328,56 @@ EXPORT Tuple * tuple_copy (const Tuple * old)
     if (old->subtunes)
         new->subtunes = g_memdup (old->subtunes, sizeof (int) * old->nsubtunes);
 
-    pthread_mutex_unlock (& mutex);
+    UNLOCK (old);
     return new;
 }
 
-/**
- * Allocates a new #Tuple structure, setting filename/URI related
- * fields based on the given filename argument by calling #tuple_set_filename.
- *
- * @param[in] filename Filename URI.
- * @return Pointer to newly allocated Tuple.
- */
-EXPORT Tuple *
-tuple_new_from_filename(const char *filename)
+EXPORT Tuple * tuple_new_from_filename (const char * filename)
 {
-    Tuple *tuple = tuple_new();
-
-    tuple_set_filename(tuple, filename);
+    Tuple * tuple = tuple_new ();
+    tuple_set_filename (tuple, filename);
     return tuple;
 }
 
-EXPORT void tuple_set_int (Tuple * tuple, int nfield, const char * field, int x)
+EXPORT void tuple_set_int (Tuple * tuple, int field, int x)
 {
-    if (nfield < 0)
-        nfield = tuple_field_by_name (field);
-    if (nfield < 0 || nfield >= TUPLE_FIELDS || tuple_fields[nfield].type != TUPLE_INT)
-        return;
+    g_return_if_fail (VALID_FIELD (field) && FIELD_TYPE (field) == TUPLE_INT);
+    LOCK (tuple);
 
-    pthread_mutex_lock (& mutex);
-
-    TupleVal * val = lookup_val (tuple, nfield, TRUE, FALSE);
+    TupleVal * val = lookup_val (tuple, field, TRUE, FALSE);
     val->x = x;
 
-    pthread_mutex_unlock (& mutex);
+    UNLOCK (tuple);
 }
 
-EXPORT void tuple_set_str (Tuple * tuple, int nfield, const char * field, const char * str)
+EXPORT void tuple_set_str (Tuple * tuple, int field, const char * str)
 {
+    g_return_if_fail (VALID_FIELD (field) && FIELD_TYPE (field) == TUPLE_STRING);
+
     if (! str)
     {
-        tuple_unset (tuple, nfield, field);
+        tuple_unset (tuple, field);
         return;
     }
 
-    if (! g_utf8_validate (str, -1, NULL))
-    {
-        fprintf (stderr, "Invalid UTF-8: %s\n", str);
-        return;
-    }
+    LOCK (tuple);
 
-    if (nfield < 0)
-        nfield = tuple_field_by_name (field);
-    if (nfield < 0 || nfield >= TUPLE_FIELDS || tuple_fields[nfield].type != TUPLE_STRING)
-        return;
+    TupleVal * val = lookup_val (tuple, field, TRUE, FALSE);
+    str_unref (val->str);
+    val->str = str_to_utf8 (str, -1);
 
-    pthread_mutex_lock (& mutex);
-
-    TupleVal * val = lookup_val (tuple, nfield, TRUE, FALSE);
-    if (val->str)
-        str_unref (val->str);
-    val->str = str_get (str);
-
-    pthread_mutex_unlock (& mutex);
+    UNLOCK (tuple);
 }
 
-EXPORT void tuple_unset (Tuple * tuple, int nfield, const char * field)
+EXPORT void tuple_unset (Tuple * tuple, int field)
 {
-    if (nfield < 0)
-        nfield = tuple_field_by_name (field);
-    if (nfield < 0 || nfield >= TUPLE_FIELDS)
-        return;
+    g_return_if_fail (VALID_FIELD (field));
+    LOCK (tuple);
 
-    pthread_mutex_lock (& mutex);
-
-    TupleVal * val = lookup_val (tuple, nfield, FALSE, TRUE);
+    TupleVal * val = lookup_val (tuple, field, FALSE, TRUE);
     if (val)
     {
-        if (tuple_fields[nfield].type == TUPLE_STRING)
+        if (tuple_fields[field].type == TUPLE_STRING)
         {
             str_unref (val->str);
             val->str = NULL;
@@ -443,96 +386,52 @@ EXPORT void tuple_unset (Tuple * tuple, int nfield, const char * field)
             val->x = 0;
     }
 
-    pthread_mutex_unlock (& mutex);
+    UNLOCK (tuple);
 }
 
-/**
- * Returns #TupleValueType of given #Tuple field.
- * Desired field can be specified either by key name or if it is
- * one of basic fields, by #TupleBasicType index.
- *
- * @param[in] tuple #Tuple structure pointer.
- * @param[in] cnfield #TupleBasicType index or -1 if key name is to be used instead.
- * @param[in] field String acting as key name or NULL if nfield is used.
- * @return #TupleValueType of the field or TUPLE_UNKNOWN if there was an error.
- */
-EXPORT TupleValueType tuple_get_value_type (const Tuple * tuple, int nfield, const char * field)
+EXPORT TupleValueType tuple_get_value_type (const Tuple * tuple, int field)
 {
-    if (nfield < 0)
-        nfield = tuple_field_by_name (field);
-    if (nfield < 0 || nfield >= TUPLE_FIELDS)
-        return TUPLE_UNKNOWN;
+    g_return_val_if_fail (VALID_FIELD (field), TUPLE_UNKNOWN);
+    LOCK (tuple);
 
-    pthread_mutex_lock (& mutex);
+    TupleVal * val = lookup_val ((Tuple *) tuple, field, FALSE, FALSE);
+    TupleValueType type = val ? FIELD_TYPE (field) : TUPLE_UNKNOWN;
 
-    TupleValueType type = TUPLE_UNKNOWN;
-
-    TupleVal * val = lookup_val ((Tuple *) tuple, nfield, FALSE, FALSE);
-    if (val)
-        type = tuple_fields[nfield].type;
-
-    pthread_mutex_unlock (& mutex);
+    UNLOCK (tuple);
     return type;
 }
 
-EXPORT char * tuple_get_str (const Tuple * tuple, int nfield, const char * field)
+EXPORT char * tuple_get_str (const Tuple * tuple, int field)
 {
-    if (nfield < 0)
-        nfield = tuple_field_by_name (field);
-    if (nfield < 0 || nfield >= TUPLE_FIELDS || tuple_fields[nfield].type != TUPLE_STRING)
-        return NULL;
+    g_return_val_if_fail (VALID_FIELD (field) && FIELD_TYPE (field) == TUPLE_STRING, NULL);
+    LOCK (tuple);
 
-    pthread_mutex_lock (& mutex);
+    TupleVal * val = lookup_val ((Tuple *) tuple, field, FALSE, FALSE);
+    char * str = val ? str_ref (val->str) : NULL;
 
-    char * str = NULL;
-
-    TupleVal * val = lookup_val ((Tuple *) tuple, nfield, FALSE, FALSE);
-    if (val)
-        str = str_ref (val->str);
-
-    pthread_mutex_unlock (& mutex);
+    UNLOCK (tuple);
     return str;
 }
 
-/**
- * Returns integer associated to #Tuple field.
- * Desired field can be specified either by key name or if it is
- * one of basic fields, by #TupleBasicType index.
- *
- * @param[in] tuple #Tuple structure pointer.
- * @param[in] cnfield #TupleBasicType index or -1 if key name is to be used instead.
- * @param[in] field String acting as key name or NULL if nfield is used.
- * @return Integer value or 0 if the field/key did not exist.
- *
- * @bug There is no way to distinguish error situations if the associated value is zero.
- */
-EXPORT int tuple_get_int (const Tuple * tuple, int nfield, const char * field)
+EXPORT int tuple_get_int (const Tuple * tuple, int field)
 {
-    if (nfield < 0)
-        nfield = tuple_field_by_name (field);
-    if (nfield < 0 || nfield >= TUPLE_FIELDS || tuple_fields[nfield].type != TUPLE_INT)
-        return 0;
+    g_return_val_if_fail (VALID_FIELD (field) && FIELD_TYPE (field) == TUPLE_INT, -1);
+    LOCK (tuple);
 
-    pthread_mutex_lock (& mutex);
+    TupleVal * val = lookup_val ((Tuple *) tuple, field, FALSE, FALSE);
+    int x = val ? val->x : -1;
 
-    int x = 0;
-
-    TupleVal * val = lookup_val ((Tuple *) tuple, nfield, FALSE, FALSE);
-    if (val)
-        x = val->x;
-
-    pthread_mutex_unlock (& mutex);
+    UNLOCK (tuple);
     return x;
 }
 
-#define APPEND(b, ...) snprintf (b + strlen (b), sizeof b - strlen (b), \
- __VA_ARGS__)
+#define APPEND(b, ...) snprintf (b + strlen (b), sizeof b - strlen (b), __VA_ARGS__)
 
 EXPORT void tuple_set_format (Tuple * t, const char * format, int chans, int rate,
  int brate)
 {
     if (format)
-        tuple_set_str (t, FIELD_CODEC, NULL, format);
+        tuple_set_str (t, FIELD_CODEC, format);
 
     char buf[32];
     buf[0] = 0;
@@ -544,8 +443,7 @@ EXPORT void tuple_set_format (Tuple * t, const char * format, int chans, int rat
         else if (chans == 2)
             APPEND (buf, _("Stereo"));
         else
-            APPEND (buf, dngettext (PACKAGE, "%d channel", "%d channels",
-             chans), chans);
+            APPEND (buf, dngettext (PACKAGE, "%d channel", "%d channels", chans), chans);
 
         if (rate > 0)
             APPEND (buf, ", ");
@@ -555,15 +453,15 @@ EXPORT void tuple_set_format (Tuple * t, const char * format, int chans, int rat
         APPEND (buf, "%d kHz", rate / 1000);
 
     if (buf[0])
-        tuple_set_str (t, FIELD_QUALITY, NULL, buf);
+        tuple_set_str (t, FIELD_QUALITY, buf);
 
     if (brate > 0)
-        tuple_set_int (t, FIELD_BITRATE, NULL, brate);
+        tuple_set_int (t, FIELD_BITRATE, brate);
 }
 
 EXPORT void tuple_set_subtunes (Tuple * tuple, int n_subtunes, const int * subtunes)
 {
-    pthread_mutex_lock (& mutex);
+    LOCK (tuple);
 
     g_free (tuple->subtunes);
     tuple->subtunes = NULL;
@@ -572,45 +470,27 @@ EXPORT void tuple_set_subtunes (Tuple * tuple, int n_subtunes, const int * subtu
     if (subtunes)
         tuple->subtunes = g_memdup (subtunes, sizeof (int) * n_subtunes);
 
-    pthread_mutex_unlock (& mutex);
+    UNLOCK (tuple);
 }
 
 EXPORT int tuple_get_n_subtunes (Tuple * tuple)
 {
-    pthread_mutex_lock (& mutex);
+    LOCK (tuple);
 
     int n_subtunes = tuple->nsubtunes;
 
-    pthread_mutex_unlock (& mutex);
+    UNLOCK (tuple);
     return n_subtunes;
 }
 
 EXPORT int tuple_get_nth_subtune (Tuple * tuple, int n)
 {
-    pthread_mutex_lock (& mutex);
+    LOCK (tuple);
 
     int subtune = -1;
     if (n >= 0 && n < tuple->nsubtunes)
         subtune = tuple->subtunes ? tuple->subtunes[n] : 1 + n;
 
-    pthread_mutex_unlock (& mutex);
+    UNLOCK (tuple);
     return subtune;
 }
-
-EXPORT char * tuple_format_title (Tuple * tuple, const char * format)
-{
-    static const gint fallbacks[] = {FIELD_TITLE, FIELD_FILE_NAME, FIELD_FILE_PATH};
-
-    char * title = tuple_formatter_process_string (tuple, format);
-
-    for (int i = 0; i < G_N_ELEMENTS (fallbacks); i ++)
-    {
-        if (title && title[0])
-            break;
-
-        str_unref (title);
-        title = tuple_get_str (tuple, fallbacks[i], NULL);
-    }
-
-    return title ? title : str_get ("");
-}
